@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # vim: set fileencoding=utf8:
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,8 +27,6 @@ import sys
 import os
 import os.path
 import shutil
-import traceback
-import textwrap
 
 from solfege import cfg
 from solfege import filesystem
@@ -49,33 +47,13 @@ if sys.platform == 'win32':
     except (IOError, os.error), e:
         print "Migration failed:", e
 
-if not os.path.exists(filesystem.app_data()):
-    os.makedirs(filesystem.app_data())
-if not os.path.exists(filesystem.user_data()):
-    os.makedirs(filesystem.user_data())
-
-try:
-    cfg.initialise("default.config", None, filesystem.rcfile())
-except UnicodeDecodeError, e:
-    traceback.print_exc()
-    print >> sys.stderr
-    print >> sys.stderr, "\n".join(textwrap.wrap(
-          "Your %s file is not properly utf8 encoded. Most likely"
-          " it is the path to some external program that contain non-ascii"
-          " characters. Please edit or delete the file. Or email it to"
-          " tca@gnu.org, and he will tell you what the problem is." % filesystem.rcfile().encode("ascii", "backslashreplace")))
-    print >> sys.stderr
-    sys.exit("I give up (solfege.py)")
-
+from solfege import presetup
+presetup.presetup("default.config", None, filesystem.rcfile())
 
 # i18n should be imported very early in program init because it setup
 # the _ and _i functions for the whole program.
-import solfege.i18n
-# MIGRATION from 2.9.2 to 2.9.3
-if cfg.get_string("app/lc_messages") == 'C (english)':
-    cfg.set_string("app/lc_messages", "C")
-solfege.i18n.setup(".", cfg.get_string("app/lc_messages"))
 
+from solfege import i18n
+i18n.setup(".", cfg.get_string("app/lc_messages"))
 import solfege.startup
-
 solfege.startup.start_app(".")
