@@ -207,7 +207,7 @@ class PianoOctaveWithAccelName(PianoKeyboard):
         if event.button == 3:
             utils.play_note(4, midi_int)
         elif event.button == 1:
-            self.m_callback(mpd.int_to_notename(midi_int))
+            self.m_callback(mpd.int_to_octave_notename(midi_int))
 
 
 class IntervalPianoWidget(PianoKeyboard):
@@ -221,30 +221,21 @@ class IntervalPianoWidget(PianoKeyboard):
         self._handle_tone_clicked(midi_int, event.button)
 
 
-class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
-    def __init__(self, exname, name, callback, sensicallback, vars_to_watch):
-        gtk.Table.__init__(self, 1, 1, True)
+class IntervalButtonsWidgetBase(gtk.VBox, cfg.ConfigUtils):
+    use_users_vocal_range = True
+
+    def __init__(self, exname, callback, sensicallback):
+        """
+        callback -- the function to call when the user click on a button.
+        sensicallback -- a function that will return a list of all interval
+                         it will be asked for.
+        """
+        gtk.VBox.__init__(self, True)
         cfg.ConfigUtils.__init__(self, exname)
-        self.m_name = name
         self.get_sensitive_buttons = sensicallback
         self.m_callback = callback
         self.m_buttons = {}
-        self.new_int_button(_("Minor Second"), 1, 0, 1, 0, 2)
-        self.new_int_button(_("Major Second"),  2, 0, 1, 2, 4)
-        self.new_int_button(_("Minor Third"),   3, 0, 1, 4, 6)
-        self.new_int_button(_("Major Third"),    4, 0, 1, 6, 8)
-        self.new_int_button(_("Perfect Fourth"),    5, 1, 2, 0, 2)
-        self.new_int_button(_("Tritone"),    6, 1, 2, 2, 4)
-        self.new_int_button(_("Perfect Fifth"),    7, 1, 2, 4, 6)
-        self.new_int_button(_("Minor Sixth"),  8, 1, 2, 6, 8)
-        self.new_int_button(_("Major Sixth"),   9, 2, 3, 0, 2)
-        self.new_int_button(_("Minor Seventh"),10, 2, 3, 2, 4)
-        self.new_int_button(_("Major Seventh"), 11, 2, 3, 4, 6)
-        self.new_int_button(_("Perfect Octave"),   12, 2, 3, 6, 8)
-        self.new_int_button(_("Minor Ninth"),  13, 3, 4, 0, 2)
-        self.new_int_button(_("Major Ninth"),   14, 3, 4, 2, 4)
-        self.new_int_button(_("Minor Tenth"), 15, 3, 4, 4, 6)
-        self.new_int_button(_("Major Tenth"),  16, 3, 4, 6, 8)
+    def post_construct(self, vars_to_watch):
         self.m_lowest_tone = mpd.LOWEST_NOTENAME
         self.m_highest_tone = mpd.HIGHEST_NOTENAME
         self.add_watch('disable_unused_intervals', self.intervals_changed)
@@ -255,24 +246,22 @@ class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
         if self.get_bool('disable_unused_intervals'):
             self.set_sensitivity(self.get_sensitive_buttons())
         else:
-            for x in range(1, 17):
-                self.m_buttons[x].set_sensitive(True)
+            self.set_sensitivity(range(mpd.interval.max_interval+1))
     def set_sensitivity(self, make_active):
-        for x in range(1, 17):
-            self.m_buttons[x].set_sensitive(x in make_active)
-    def new_int_button(self, txt, nr, x1, x2, y1, y2):
+        for x in range(1, mpd.interval.max_interval + 1):
+            if x in self.m_buttons:
+                self.m_buttons[x].set_sensitive(x in make_active)
+    def mk_button(self, txt, nr):
         # buttonwidget calls m_callback with None as midi_int because it
         # does not know if you mean interval up or down when you click
         # the buttons
         self.m_buttons[nr] = b = gtk.Button(txt)
-        w, h = b.size_request()
-        b.set_size_request(w, int(h*1.4))
         l=b.get_child().set_justify(gtk.JUSTIFY_CENTER)
-        self.attach(b, x1, x2, y1, y2)
         b.connect('clicked',
                   lambda s, nr=nr, self=self:self.m_callback(1, nr, None))
         b.set_data('interval', nr)
         b.connect('event', self._abc)
+        return b
     def _abc(self, button, event):
         if event.type == gtk.gdk.BUTTON_RELEASE and event.button == 3:
             self.m_callback(3, button.get_data('interval'), None)
@@ -295,6 +284,81 @@ class IntervalButtonsWidget(gtk.Table, cfg.ConfigUtils):
         """
         pass
 
+
+class IntervalButtonsWidget(IntervalButtonsWidgetBase):
+    def __init__(self, exname, callback, sensicallback, vars_to_watch):
+        IntervalButtonsWidgetBase.__init__(self, exname, callback, sensicallback)
+        self.g_table = gtk.Table()
+        self.pack_start(self.g_table)
+        self.new_int_button(_("Minor Second"),   1, 0, 1, 0, 1)
+        self.new_int_button(_("Major Second"),   2, 1, 2, 0, 1)
+        self.new_int_button(_("Minor Third"),    3, 2, 3, 0, 1)
+        self.new_int_button(_("Major Third"),    4, 3, 4, 0, 1)
+        self.new_int_button(_("Perfect Fourth"), 5, 0, 1, 1, 2)
+        self.new_int_button(_("Tritone"),        6, 1, 2, 1, 2)
+        self.new_int_button(_("Perfect Fifth"),  7, 2, 3, 1, 2)
+        self.new_int_button(_("Minor Sixth"),    8, 3, 4, 1, 2)
+        self.new_int_button(_("Major Sixth"),     9, 0, 1, 2, 3)
+        self.new_int_button(_("Minor Seventh"),  10, 1, 2, 2, 3)
+        self.new_int_button(_("Major Seventh"),  11, 2, 3, 2, 3)
+        self.new_int_button(_("Perfect Octave"), 12, 3, 4, 2, 3)
+        self.new_int_button(_("Minor Ninth"),    13, 0, 1, 3, 4)
+        self.new_int_button(_("Major Ninth"),    14, 1, 2, 3, 4)
+        self.new_int_button(_("Minor Tenth"),    15, 2, 3, 3, 4)
+        self.new_int_button(_("Major Tenth"),    16, 3, 4, 3, 4)
+        self.new_int_button(_("Perfect 11th"),     17, 0, 1, 4, 5)
+        self.new_int_button(_("Octave + Tritone"), 18, 1, 2, 4, 5)
+        self.new_int_button(_("Perfect 12th"),     19, 2, 3, 4, 5)
+        self.new_int_button(_("Minor 13th"),       20, 3, 4, 4, 5)
+        self.new_int_button(_("Major 13th"),    21, 0, 1, 5, 6)
+        self.new_int_button(_("Minor 14th"),    22, 1, 2, 5, 6)
+        self.new_int_button(_("Major 14th"),    23, 2, 3, 5, 6)
+        self.new_int_button(_("Perfect 15th"),  24, 3, 4, 5, 6)
+        self.post_construct(vars_to_watch)
+    def new_int_button(self, txt, nr, x1, x2, y1, y2):
+        b = self.mk_button(txt, nr)
+        w, h = b.size_request()
+        b.set_size_request(w, int(h*1.4))
+        self.g_table.attach(b, x1, x2, y1, y2)
+
+
+class IntervalButtonsWidget2(IntervalButtonsWidgetBase):
+    def __init__(self, exname, callback, sensicallback, vars_to_watch):
+        IntervalButtonsWidgetBase.__init__(self, exname, callback, sensicallback)
+        self.row(((_("Minor Second"), 1),
+                  (_("Major Second"), 2),
+                  (_("Minor Third"), 3),
+                  (_("Major Third"), 4)))
+        self.row(((_("Perfect Fourth"), 5),
+                  (_("Tritone"), 6),
+                  (_("Perfect Fifth"), 7)))
+        self.row(((_("Minor Sixth"), 8),
+                  (_("Major Sixth"), 9),
+                  (_("Minor Seventh"), 10),
+                  (_("Major Seventh"), 11)))
+        self.row(((_("Perfect Octave"), 12),))
+        self.row(((_("Minor Ninth"), 13),
+                  (_("Major Ninth"), 14),
+                  (_("Minor Tenth"), 15),
+                  (_("Major Tenth"), 16),))
+        self.row(((_("Perfect 11th"), 17),
+                  (_("Octave + Tritone"), 18),
+                  (_("Perfect 12th"), 19),))
+        self.row(((_("Minor 13th"), 20),
+                  (_("Major 13th"), 21),
+                  (_("Major 14th"), 22),
+                  (_("Minor 14th"), 23),))
+        self.row(((_("Perfect 15th"), 24),))
+        self.post_construct(vars_to_watch)
+
+    def row(self, data):
+        hbox = gtk.HBox(True)
+        self.pack_start(hbox)
+        for label, i in data:
+            b = self.mk_button(label, i)
+            hbox.pack_start(b)
+
+
 class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
     def __init__(self, callback, strings,
                 string_thickness=(1, 1, 1, 1, 1, 1)):
@@ -309,9 +373,9 @@ class AbstractGuitarWidget(gtk.DrawingArea, CairoCommon):
         self.m_neckborder = 6
         self.m_neckl = 0
         self.m_xlist = []
-        self.m_lowest_tone = mpd.int_to_notename(
+        self.m_lowest_tone = mpd.int_to_octave_notename(
             min(map(mpd.notename_to_int, self.m_strings)))
-        self.m_highest_tone = mpd.int_to_notename(
+        self.m_highest_tone = mpd.int_to_octave_notename(
             len(self.m_fretdist) - 1 + max(map(mpd.notename_to_int, self.m_strings)))
         self.m_stringtuning = map(mpd.notename_to_int, self.m_strings)
         tmp = 0
@@ -503,58 +567,23 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
         gtk.DrawingArea.__init__(self)
         CairoCommon.__init__(self)
         self.m_callback = callback
-        self.m_notenames_norwegian = (("g,", "bes,", "des", "e",
-                 "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''", "d''''"),
-                ("fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"))
-        self.m_notenames_swedish = (
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''"),
-                ("dis,", "fis,", "a,", "c", "ees", "fis", "a", "c'",
-                 "ees'", "fis'", "a'", "c''", "ees''", "fis''", "a''",
-                 "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                 )
-        self.m_notenames_finnish = (
-                ("dis,", "fis,", "a,", "c", "ees", "fis", "a", "c'",
-                 "ees'", "fis'", "a'", "c''", "ees''", "fis''", "a''",
-                 "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("d,", "f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''"),
-                ("e,", "g,", "bes,", "des",
-                 "e", "g", "bes", "des'", "e'", "g'", "bes'", "des''", "e''",
-                 "g''", "bes''", "des'''", "e'''", "g'''", "bes'''", "des''''"),
-                ("dis,", "fis,", "a,", "c", "ees",
-                 "fis", "a", "c'", "ees'", "fis'", "a'", "c''", "ees''",
-                 "fis''", "a''", "c'''", "ees'''", "fis'''", "a'''", "c''''"),
-                ("f,", "aes,", "b,", "d",
-                 "f", "aes", "b", "d'", "f'", "aes'", "b'", "d''", "f''",
-                 "aes''", "b''", "d'''", "f'''", "aes'''", "b'''", "d''''"),
-                 )
-        self.m_notenames = {'norwegian': self.m_notenames_norwegian,
-                            'swedish': self.m_notenames_swedish,
-                       'finnish': self.m_notenames_finnish}[keyboard_system]
+        def ff(first, count):
+            i = mpd.notename_to_int(first)
+            return [mpd.int_to_octave_notename(i + x * 3) for x in range(count)]
+        keyboard_def = {'norwegian':
+            (ff("g,", 19), ff("f,", 20), ff("fis,", 19),
+            ff("e,", 20), ff("f,", 19))}
+        keyboard_def['swedish'] = (
+            ff("e,", 19), ff("dis,", 20), ff("f,", 19),
+            ff("e,", 20), ff("fis,", 19))
+        keyboard_def['finnish'] = (
+            ff("dis,", 19), ff("d,", 20), ff("e,", 19),
+            ff("dis,", 20), ff("f,", 19))
+        keyboard_def['belgian'] = (
+            ff("fis,", 19), ff("e,", 20), ff("f,", 19),
+            ff("es,", 20), ff("e,", 19)
+        )
+        self.m_notenames = keyboard_def[keyboard_system]
         self.m_lowest_tone = mpd.HIGHEST_NOTENAME
         self.m_highest_tone = mpd.LOWEST_NOTENAME
         for v in self.m_notenames:
@@ -655,32 +684,41 @@ class AccordionWidget(gtk.DrawingArea, CairoCommon):
 
 
 inputwidget_names = [_("Buttons"),
+                 _("Buttons %i") % 2,
                  _("Piano"), _("Guitar"), _("Bass"),
                  _("5 string bass"), _("6 string bass"),
                  _("Accordion B griff"), _("Accordion C griff"),
-                 _("Accordion (system used in Finland)")]
+                 _("Accordion (system used in Finland)"),
+                 _("Accordion (Belgian Charleroi do2)")
+                 ]
 
-def name_to_inputwidget(name, callback):
-    if name == _("Piano"):
+def int_to_inputwidget(i, callback, exname, get_interval_input_list, vars_to_watch):
+    if i in (0, 1):
+        w = {0: IntervalButtonsWidget,
+             1: IntervalButtonsWidget2}[i](exname, callback,
+                                           get_interval_input_list,
+                                           vars_to_watch)
+    elif i == 2:
         w = IntervalPianoWidget(callback)
-    elif name == _("Accordion B griff"):
-        w = AccordionWidget(callback, 'norwegian')
-    elif name == _("Accordion C griff"):
-        w = AccordionWidget(callback, 'swedish')
-    elif name == _("Accordion (system used in Finland)"):
-        w = AccordionWidget(callback, 'finnish')
-    elif name == _("Guitar"):
+    elif i == 3:
         w = GuitarWidget(callback, ["e'", "b", "g", "d", "a,", "e,"],
                                    (1, 1, 1, 2, 2, 3))
-    elif name == _("Bass"):
+    elif i == 4:
         w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,"],
                                  (1, 1, 2, 3))
-    elif name == _("5 string bass"):
+    elif i == 5:
         w = AbstractGuitarWidget(callback, ["g,", "d,", "a,,", "e,,", "b,,,"],
                                  (1, 1, 2, 3, 3))
-    else:
-        assert name == _("6 string bass")
+    elif i == 6:
         w = AbstractGuitarWidget(callback, ["c", "g,", "d,", "a,,", "e,,", "b,,,"],
                                  (1, 1, 2, 2, 3, 3))
+    elif i == 7:
+        w = AccordionWidget(callback, 'norwegian')
+    elif i == 8:
+        w = AccordionWidget(callback, 'swedish')
+    elif i == 9:
+        w = AccordionWidget(callback, 'finnish')
+    elif i == 10:
+        w = AccordionWidget(callback, 'belgian')
     return w
 
